require 'active_record'

include ActiveRecord::Tasks

root = File.expand_path '../../..', __FILE__

DatabaseTasks.tap do |t|
  Application.start(:mysql)

  t.env = Application.env.to_s
  t.database_configuration = Application["mysql.config"]
  t.root = root
end

namespace :eng20732 do
  namespace :migrate do
    namespace :customer do

      require 'activerecord-import'
      require 'role_management/stores'
      require 'role_management/models'
      require 'role_management/stores/role_documents_directory'
      
      # NOTE: Only role_doc_id and organization_id are used, the other fields are informational
      orgs = [
        { :role_doc_id => 13, :organization_id => 137438953498,  :name => "Test",                              :tenant_org => "MSP Demo" },
        { :role_doc_id => 19, :organization_id => 5978594478012, :name=> "Test_Role_Document",                 :tenant_org => "IHS Markit"},
        { :role_doc_id => 21, :organization_id => 137438953481,  :name=> "AWS Power User",                     :tenant_org => "Master All Accounts"},
        { :role_doc_id => 22, :organization_id => 893353197629,  :name=> "Very . Limited .",                   :tenant_org => "VMWare Demo"},
        { :role_doc_id => 26, :organization_id => 7902739824812, :name=> "Chelton's Test Role Document",       :tenant_org => "Smidgets Widgets II"},
        { :role_doc_id => 29, :organization_id => 7902739824812, :name=> "Chelton's second test RD",           :tenant_org => "Smidgets Widgets II"},
        { :role_doc_id => 41, :organization_id => 9070970929271, :name=> "Toyota Administrator",               :tenant_org => "Toyota Connected Master"},
        { :role_doc_id => 57, :organization_id => 893353197629,  :name=> "FlexOrgs Migration RD",              :tenant_org => "VMWare Demo"},
        { :role_doc_id => 62, :organization_id => 893353198203,  :name=> "Standard AWS/Azure User",            :tenant_org => "Global Costs"},
        { :role_doc_id => 66, :organization_id => 893353197629,  :name=> "Customer Admin",                     :tenant_org => "VMWare Demo"},
        { :role_doc_id => 67, :organization_id => 893353197629,  :name=> "Äzure Röle öf Pöwer!",               :tenant_org => "VMWare Demo"},
        { :role_doc_id => 71, :organization_id => 893353197629,  :name=> "User admin",                         :tenant_org => "VMWare Demo"},
        { :role_doc_id => 72, :organization_id => 24,            :name=> "CHT-Role-Doc",                       :tenant_org => "Cloud Health Technologies"},
        { :role_doc_id => 73, :organization_id => 9070970929271, :name=> "Toyota Standard User",               :tenant_org => "Toyota Connected Master"},
        { :role_doc_id => 74, :organization_id => 2680059592853, :name=> "Api Access",                         :tenant_org => "Cisco Enterprise"},
        { :role_doc_id => 76, :organization_id => 893353197629,  :name=> "Copy of Standard User - Test ",      :tenant_org => "VMWare Demo"},
        { :role_doc_id => 77, :organization_id => 893353197629,  :name=> "Copy of Standard User-profile test", :tenant_org => "VMWare Demo"},
        { :role_doc_id => 78, :organization_id => 137438953481,  :name=> "Power User w/AWS tagging disable",   :tenant_org => "Master All Accounts"},
        { :role_doc_id => 81, :organization_id => 24, :name=> "test_role_document_full access",                :tenant_org => "Cloud Health Technologies"},
        { :role_doc_id => 82, :organization_id => 24, :name=> "test_role_document_full access",                :tenant_org => "Cloud Health Technologies"}                                                                            
      ]

      desc "Migrate specific customer role documents to add an organization id for ENG-20732"
      task :organization_ids_up do
          puts "Migrating Organization Ids UP"
          ActiveRecord::Base.transaction do
            orgs.each do |org|
              update_organization_id(::RoleManagement::Stores::RoleDocument.where(id: org[:role_doc_id]).first, org[:organization_id])
            end
          end
      end
      task :organization_ids_down do
        puts "Migrating Organization Ids DOWN"
        ActiveRecord::Base.transaction do
          orgs.each do |org|
            update_organization_id(::RoleManagement::Stores::RoleDocument.where(id: org[:role_doc_id]).first,nil)
          end
        end
      end
      def update_organization_id(role_doc, organization_id)
        # In the event the role document goes away before the migration
        return if role_doc.nil?
        role_doc.organization_id = organization_id
        role_doc.save!
      end
    end
  end
end