# frozen_string_literal: true

namespace :eng48854 do
  namespace :get_ugrbs_with_custom_docs_system_defined_role_docs do
    desc "Get count of ugrbs with custom role documents and system defined role documents"
    task :list_ugrbs do
      logger = Application['logger']
      require 'activerecord-import'
      require 'role_management/stores'
      require 'role_management/stores/role_documents_directory'
      require 'role_management/models'
      require 'csv'
      include ::RoleManagement::Stores

      custom_role_documents = []
      system_defined_role_documents = []
      custom_role_documents_ugrbs = 0
      system_defined_role_documents_documents_ugrbs = 0
      system_defined_ugrbs_customers = 0
      custom_defined_ugrbs_customers = 0
      system_defined_ugrbs_customers_csv = []
      custom_defined_ugrbs_customers_csv = []
      custom_role_documents_ugrbs = 0

      logger.info("Getting custom role documents and system defined role documents")
      puts("Getting custom role documents and system defined role documents")
      ActiveRecord::Base.transaction do
        custom_role_documents << RoleManagement::Stores::RoleDocument.where(system_defined: false).pluck(:id)
        system_defined_role_documents << RoleManagement::Stores::RoleDocument.where(system_defined: true).pluck(:id)
      end

      if !custom_role_documents.to_a.join(",").nil? && !custom_role_documents.to_a.join(",").empty?
        ActiveRecord::Base.transaction do
          sql = <<-SQL
              select count(*) as custom_ugrbs from user_group_role_bindings where role_document_id IN (#{custom_role_documents.to_a.join(",")})
          SQL
          custom_role_documents_ugrbs = ActiveRecord::Base.connection.execute(sql.chomp)
        end
      end
      if !system_defined_role_documents.to_a.join(",").nil? && !system_defined_role_documents.to_a.join(",").empty?
        ActiveRecord::Base.transaction do
          sql = <<-SQL
              select count(*) as system_ugrbs from user_group_role_bindings where role_document_id IN (#{system_defined_role_documents.to_a.join(",")})
          SQL
          system_defined_role_documents_documents_ugrbs = ActiveRecord::Base.connection.execute(sql.chomp)
        end
      end
      if system_defined_role_documents_documents_ugrbs.first.nil? ||system_defined_role_documents_documents_ugrbs.first.empty?
        logger.info("Found [0] system defined role documents ugrbs")
        puts("Found [0] system defined role documents ugrbs")
      else
        logger.info("Found [#{system_defined_role_documents_documents_ugrbs.first["system_ugrbs"]}] system defined role documents ugrbs")
        puts("Found [#{system_defined_role_documents_documents_ugrbs.first["system_ugrbs"]}] system defined role documents ugrbs")
      end

      if custom_role_documents_ugrbs == 0
        logger.info("Found #{custom_role_documents_ugrbs} custom role documents ugrbs")
        puts("Found #{custom_role_documents_ugrbs} custom role documents ugrbs")
      else
        if custom_role_documents_ugrbs.first.nil? ||custom_role_documents_ugrbs.first.empty?
          logger.info("Found [0] custom role documents ugrbs")
          puts("Found [0] custom role documents ugrbs")
        else
          logger.info("Found [#{custom_role_documents_ugrbs.first["custom_ugrbs"]}] custom role documents ugrbs")
          puts("Found [#{custom_role_documents_ugrbs.first["custom_ugrbs"]}] custom role documents ugrbs")
        end
      end

      puts "Get customers which contain only system defined ugrbs"
      ActiveRecord::Base.transaction do
        sql = <<-SQL
              SELECT count(DISTINCT customer_crn) as system_customers FROM user_group_role_bindings WHERE customer_crn NOT IN (SELECT DISTINCT customer_crn
                FROM user_group_role_bindings
                WHERE role_document_id not in (1,2,3,4)) AND role_document_id in (1,2,3,4);
        SQL
        system_defined_ugrbs_customers = ActiveRecord::Base.connection.execute(sql.chomp)
      end
      if system_defined_ugrbs_customers.first.nil? ||system_defined_ugrbs_customers.first.empty?
        logger.info("Found [0] customer with system defined role documents ugrbs")
        puts("Found [0] customers with system defined role documents ugrbs")
      else
        logger.info("Found [#{system_defined_ugrbs_customers.first["system_customers"]}] customers with system defined role documents ugrbs")
        puts("Found [#{system_defined_ugrbs_customers.first["system_customers"]}] customers with system defined role documents ugrbs")
      end



      puts "Get customers which contain only custom ugrbs"
      ActiveRecord::Base.transaction do
        sql = <<-SQL
              SELECT count(DISTINCT customer_crn) as custom_customers FROM user_group_role_bindings WHERE customer_crn NOT IN (SELECT DISTINCT customer_crn
                FROM user_group_role_bindings
                WHERE role_document_id in (1,2,3,4)) AND role_document_id not in (1,2,3,4);
        SQL
        custom_defined_ugrbs_customers = ActiveRecord::Base.connection.execute(sql.chomp)
      end

      if custom_defined_ugrbs_customers.first.nil? ||custom_defined_ugrbs_customers.first.empty?
        logger.info("Found [0] customer with custom defined role documents ugrbs")
        puts("Found [0] customers with custom defined role documents ugrbs")
      else
        logger.info("Found [#{custom_defined_ugrbs_customers.first["custom_customers"]}] customers with custom defined role documents ugrbs")
        puts("Found [#{custom_defined_ugrbs_customers.first["custom_customers"]}] customers with custom defined role documents ugrbs")
      end

      puts "Get customers which contain only system defined ugrbs"
      ActiveRecord::Base.transaction do
        sql = <<-SQL
              SELECT DISTINCT customer_crn FROM user_group_role_bindings WHERE customer_crn NOT IN (SELECT DISTINCT customer_crn
                FROM user_group_role_bindings
                WHERE role_document_id not in (1,2,3,4)) AND role_document_id in (1,2,3,4);
        SQL
        system_defined_ugrbs_customers_csv = ActiveRecord::Base.connection.execute(sql.chomp)
      end
      CSV.open("/tmp/system_defined_ugrbs_customers.csv", "wb") do |csv|
        csv << ["Customer_Tenant"]
        if !system_defined_ugrbs_customers_csv.first.nil? && !system_defined_ugrbs_customers_csv.first.empty?
          system_defined_ugrbs_customers_csv.each do |row|
              csv << [row['customer_crn']]
          end
        end
      end

      puts "Get customers which contain only custom ugrbs"
      ActiveRecord::Base.transaction do
        sql = <<-SQL
              SELECT DISTINCT customer_crn FROM user_group_role_bindings WHERE customer_crn NOT IN (SELECT DISTINCT customer_crn
                FROM user_group_role_bindings
                WHERE role_document_id in (1,2,3,4)) AND role_document_id not in (1,2,3,4);
        SQL
        custom_defined_ugrbs_customers_csv = ActiveRecord::Base.connection.execute(sql.chomp)
      end

      CSV.open("/tmp/custom_defined_ugrbs_customers.csv", "wb") do |csv|
        csv << ["Customer_Tenant"]
        if !custom_defined_ugrbs_customers_csv.first.nil? && !custom_defined_ugrbs_customers_csv.first.empty?
          custom_defined_ugrbs_customers_csv.each do |row|
            csv << [row['customer_crn']]
          end
        end
      end

    end
  end
end